//+----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// FileName: AutoLock.h
// Purpose:  Multithreaded object wrappers for process object locking
//
// Contains:
//			AutoLock Class
//				CAutoLockT		-- Helper class used to Auto-Lock/Unlock
//								   classes derived from CLockableXXX classes
//								   listed below or from ATL's CComObjectRoot
//								   class.
//
//			Lock Classes
//				CLockableCS		-- Critical Section base class
//	History:
//	[0]		12-15-2004	Arjay -- Method Created
//-----------------------------------------------------------------------------
//+----------------------------------------------------------------------------
#pragma once

//+----------------------------------------------------------------------------
// Class:   CAutoLockT< lockableclass >
//
// Purpose: Helper class used to lock a LockableObject [derived] class
//      on construction and unlock it on destruction.  The indicated object
//		will be locked from the time the CAutoLock object is created until
//		the CAutoLock object goes out of scope (or is otherwise destroyed).
//		The class supplied as a template argument must support accessible
//		functions named Lock() and Unlock().  
//
//		For example, anything derived from CLockableXXX classes below, or
//		from ATL's CComObjectRoot is autolockable.
//
// Constructors:
//		CAutoLockT( T )				-- Critical Section ctor
//+----------------------------------------------------------------------------
template<class T>
class CAutoLockT
{
// Attributes
private:
	T* m_pObject;		// the locked object

// Ctor/dtor
public:
	// Critical Section Ctor
	inline CAutoLockT( T* pObject ) throw()
		: m_pObject(pObject)
	{
		ASSERT( NULL != pObject );
		m_pObject->Lock();
	}

	// dtor
	inline ~CAutoLockT()
	{
		m_pObject->Unlock();
	}
};

//+----------------------------------------------------------------------------
// Class:   CLockableCS
//
// Purpose: This class is intended to be used as a base class for a 
//          multithreaded Critical Section lockable object. 
//
//+----------------------------------------------------------------------------
class CLockableCS
{
// Methods
public:
	// Ctor / Dtor
	inline CLockableCS() throw()	{ InitializeCriticalSection( &m_CS ); }
	inline ~CLockableCS() throw()	{ DeleteCriticalSection( &m_CS ); }

	// Lock / Unlock
	inline void Lock() throw()		{ EnterCriticalSection( &m_CS ); }
	inline void Unlock() throw()	{ LeaveCriticalSection( &m_CS ); }

// Attributes
private:
	CRITICAL_SECTION m_CS;		// Internal Critical Section
};